@@ -1,6 +1,6 @@
 import numpy as np
 import torch
-from kpt.model.quaternion import qmul, qrot
+from pytorch3d.transforms import matrix_to_quaternion, quaternion_to_matrix
 
 
 class TorchSkeleton:
@@ -42,7 +42,7 @@ class TorchSkeleton:
             if not joint_info['children']: # If joint is at edge, do not add to kinematic_chain
                 continue
 
-            joint_info['offsets'] = np.array(joint_info['offsets']).reshape(3, 1)
+            joint_info['offsets'] = torch.Tensor(joint_info['offsets']).reshape(3, 1)
 
             if 'channels' in joint_info:
                 if joint_name is self.root_name:
@@ -56,8 +56,8 @@ class TorchSkeleton:
 
         offsets = []
         for joint in kinematic_chain:
-            offsets.append(kinematic_chain[joint]['offsets'].squeeze())
-        self._offset_arr = torch.Tensor(offsets).to(self.device)
+            offsets.append(kinematic_chain[joint]['offsets'].squeeze().numpy())
+        self._offset_arr = torch.Tensor(np.array(offsets).squeeze()).to(self.device)
         return kinematic_chain
 
     def forward_kinematics(self, root_positions: torch.Tensor, rotations: torch.Tensor, rot_repr='quaternion'):
@@ -73,9 +73,6 @@ class TorchSkeleton:
         """    
         position_arr_w = []
         rotation_arr_w = []
-
-        expanded_offsets = self.offset_arr.expand(rotations.shape[0], self.offset_arr.shape[0], self.offset_arr.shape[1])
-
         for joint in self.joints:
             joint_idx = self.joints.index(joint)
             if not self.skeleton[joint]['parent']: # If joint is root
@@ -83,8 +80,9 @@ class TorchSkeleton:
                 rotation_arr_w.append(rotations[:,0,:])
             else:
                 parent_idx = self.joints.index(self.skeleton[joint]['parent'])
-                position_arr_w.append(qrot(rotation_arr_w[parent_idx], expanded_offsets[:, joint_idx]) + position_arr_w[parent_idx])
-                rotation_arr_w.append(qmul(rotation_arr_w[parent_idx], rotations[:, joint_idx, :]))
+                position_arr_w.append(torch.matmul(quaternion_to_matrix(rotation_arr_w[parent_idx]), self.offset_arr[joint_idx, :]) + position_arr_w[parent_idx])
+                rotation_arr_w.append(matrix_to_quaternion(torch.matmul(quaternion_to_matrix(rotation_arr_w[parent_idx]), quaternion_to_matrix(rotations[:, joint_idx, :]))))  
         position_tensor_w = torch.stack(position_arr_w, dim=2).permute(0,2,1)
-        return position_tensor_w
+        rotation_tensor_w = torch.stack(rotation_arr_w, dim=2).permute(0,2,1)
+        return position_tensor_w, rotation_tensor_w
 
